/* 
 * JCM kernel instrumentation module
 * Copyright(C) 2013 Jon Masters <jcm@jonmasters.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kprobes.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE == KERNEL_VERSION(3, 6, 11)
void (*show_regs_fn)(struct pt_regs * regs) = (void *)0xc000f9a8;
#elif LINUX_VERSION_CODE == KERNEL_VERSION(3, 6, 10)
void (*show_regs_fn)(struct pt_regs * regs) = (void *)0xc000f368;
#else
#error "unsupported kernel"
#endif

static int jp_func_die(const char *str, struct pt_regs *regs, int err)
{
	printk("JCM_DEBUG: die(str: %s, regs: 0x%08x, err: %d\n",
		str, (unsigned int)regs, err);

	show_regs_fn(regs);

	jprobe_return();
	return 0;
}

static struct jprobe jp_die = {
	.entry = jp_func_die,
#if LINUX_VERSION_CODE == KERNEL_VERSION(3, 6, 11)
	.kp.addr = (kprobe_opcode_t *) 0xc001233c,
#elif LINUX_VERSION_CODE == KERNEL_VERSION(3, 6, 10)
	.kp.addr = (kprobe_opcode_t *) 0xc0011d74,
#else
#error "unsupported kernel"
#endif
};

int init_module(void)
{
	register_jprobe(&jp_die);
	printk("JCM_DEBUG: registered jprobe at die\n");
	return 0;
}

void exit_module(void)
{
	unregister_jprobe(&jp_die);
	printk("JCM_DEBUG: unregistered probe at die\n");
}

MODULE_AUTHOR("Jon Masters");
MODULE_DESCRIPTION("JCM kernel instrumentation module");
MODULE_LICENSE("GPL");

